/* Copyright 2000 Red Hat, Inc.
 *
 * Than Ngo <than@redhat.de>
 *
 * This software may be freely redistributed under the terms of the GNU
 * public license.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


#ifdef QTONLY
#include "kde-int.h"
#include <unistd.h>
#include <stdlib.h>
#include <qapp.h>
#include <qstring.h>
/* #include <qwindowsstyle.h> */

#ifndef DATADIR
#  define DATADIR "/usr/share"
#endif

static KIconLoader kil;

QPixmap KIconLoader::loadIcon(const char *file)
{
  QString datadir(DATADIR "/icons/");

  QPixmap pix(datadir + file);

  if (pix.isNull())
    pix.load(datadir + file + ".xpm");
  
  if (pix.isNull())
    pix.load(datadir + file + ".png");
  
  if (pix.isNull())
    pix.load(datadir + "mini/" + file + ".xpm");

  if (pix.isNull())
    pix.load(datadir + "mini/" + file + ".png");

  datadir = DATADIR "/toolbar/";

  if (pix.isNull())
    pix.load(datadir + file + ".xpm");

  if (pix.isNull())
    pix.load(datadir + file + ".png");

  return pix;
}

KIconLoader *KGlobal::iconLoader()
{
  return &kil;
}


void invokeHTMLHelp(QString filename)
{
  QApplication::flushX();

  if (fork() == 0) {
    // first try the locale setting
    QString file(filename);

    if (file.isNull()) {
      warning("no help file %s found\n", filename.ascii());
      _exit(1);
    }
    
    /* Since this is a library, we must conside the possibilty that
     * we are being used by a suid root program. These next two
     * lines drop all privileges.
     */
    setuid(getuid());
    setgid(getgid());
    
    const char *shell = "/bin/sh";
    if (getenv("SHELL"))
      shell = getenv("SHELL");
    
    file.prepend("isdnview ");
    
    execl(shell, shell, "-c", file.ascii(), 0L);
    _exit(1);
  }
}

#include <qframe.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include "kcdialog.h"

KCModule::KCModule(QWidget * parent, const char *name)
  :QWidget(parent, name), _btn(Help | Default | Reset | Cancel | Apply | Ok)
{
}

void setVisible(QPushButton * btn, bool vis)
{
  if (vis)
    btn->show();
  else
    btn->hide();
}

KCDialog::KCDialog(KCModule * client, QWidget * parent, const char *name,
		   bool modal, WFlags f)
:QDialog(parent, name, modal, f), _client(client)
{
  client->reparent(this, 0, QPoint(0, 0), true);
  
  _sep = new QFrame(this);
  _sep->setFrameStyle(QFrame::HLine | QFrame::Sunken);
  _sep->show();

  _help = new QPushButton(i18n("Help"), this);
  _default = new QPushButton(i18n("Default"), this);
  _reset = new QPushButton(i18n("Reset"), this);
  _cancel = new QPushButton(i18n("Cancel"), this);
  _apply = new QPushButton(i18n("Apply"), this);
  _ok = new QPushButton(i18n("Ok"), this);

  // only enable the requested buttons
  int b = _client->buttons();
  setVisible(_help, b & KCModule::Help);
  setVisible(_default, b & KCModule::Default);
  setVisible(_reset, b & KCModule::Reset);
  setVisible(_cancel, b & KCModule::Cancel);
  setVisible(_apply, b & KCModule::Apply);
  setVisible(_ok, b & KCModule::Ok);
  
  // disable initial buttons
  _reset->setEnabled(false);
  _apply->setEnabled(false);

  connect(_help, SIGNAL(clicked()), this, SLOT(helpClicked()));
  connect(_default, SIGNAL(clicked()), this, SLOT(defaultClicked()));
  connect(_reset, SIGNAL(clicked()), this, SLOT(resetClicked()));
  connect(_cancel, SIGNAL(clicked()), this, SLOT(cancelClicked()));
  connect(_apply, SIGNAL(clicked()), this, SLOT(applyClicked()));
  connect(_ok, SIGNAL(clicked()), this, SLOT(okClicked()));
  clientChanged(false);
  
  connect(client, SIGNAL(changed(bool)), this,
          SLOT(clientChanged(bool)));

  QGridLayout *top = new QGridLayout(this, 4, 6, 5);
  top->addMultiCellWidget(client, 0, 0, 0, 6);
  top->addMultiCellWidget(_sep, 1, 1, 0, 6);
  top->addWidget(_help, 2, 0);
  top->addWidget(_default, 2, 1);
  top->addWidget(_reset, 2, 2);
  top->addWidget(_apply, 2, 4);
  top->addWidget(_ok, 2, 5);
  top->addWidget(_cancel, 2, 6);
  
  top->setRowStretch(0, 1);
  top->setColStretch(3, 1);
  
  top->activate();
}

void KCDialog::helpClicked()
{
  const char *lang = "en";
  if (getenv("LANG"))
    lang = getenv("LANG");
  if (getenv("LANGUAGE"))
    lang = getenv("LANGUAGE");

  QString helpfile = QString(DOCDIR "/HTML/") + QString(lang) +
    QString("/" PACKAGE "/index.html");
  QFile f(helpfile.ascii());
  if (!f.exists())
    helpfile = QString(DOCDIR "/HTML/en/") + QString(PACKAGE "/index.html");
  
  invokeHTMLHelp(helpfile.ascii());
}

void KCDialog::defaultClicked()
{
  _client->defaults();
  clientChanged(true);
}

void KCDialog::resetClicked()
{
  _client->load();
  clientChanged(false);
}


void KCDialog::cancelClicked()
{
//  reject();
 exit(0);
}


void KCDialog::applyClicked()
{
  _client->save();
  clientChanged(false);
}


void KCDialog::okClicked()
{
  _client->save();
  accept();
}

void KCDialog::clientChanged(bool state)
{
  // enable/disable buttons
  // int b = _client->buttons();
  _reset->setEnabled(state);
  _apply->setEnabled(state);
  _ok->setEnabled(state);
}

template class QList < KButtonBox::KButtonBoxItem >;

KButtonBox::KButtonBox(QWidget * parent, int _orientation,
		       int border, int autoborder):QWidget(parent)
{
  orientation = _orientation;
  _border = border;
  if (autoborder < 0)
    _autoborder = border;
  else
    _autoborder = autoborder;
  
  buttons.setAutoDelete(TRUE);
}

KButtonBox::~KButtonBox()
{
}

QPushButton *KButtonBox::addButton(const QString & text, bool noexpand)
{
  KButtonBoxItem *item = new KButtonBoxItem;
    
  item->button = new QPushButton(text, this);
  item->noexpand = noexpand;
  buttons.append(item);
  item->button->adjustSize();

  return item->button;
}

void KButtonBox::addStretch(int scale)
{
  if (scale > 0) {
    KButtonBoxItem *item = new KButtonBoxItem;
    item->button = 0;
    item->noexpand = FALSE;
    item->stretch = scale;
    buttons.append(item);
  }
}

void KButtonBox::layout()
{
  // resize all buttons
  QSize bs = bestButtonSize();

  for (unsigned int i = 0; i < buttons.count(); i++) {
    KButtonBoxItem *item = buttons.at(i);
    QPushButton *b = item->button;
    if (b != 0) {
      if (!item->noexpand) {
/*        if (b->style() == MotifStyle && b->isDefault()) { */
        if (0) {
          QSize s = bs;
          s.setWidth(bs.width() + extraMotifWidth);
          s.setHeight(bs.height() + extraMotifHeight);
          b->setFixedSize(s);
          b->setDefault(TRUE);
        } else {
          b->setFixedSize(bs);
        }
      } else
        b->setFixedSize(buttonSizeHint(b));
    }
  }
  
  setMinimumSize(sizeHint());
}

void KButtonBox::placeButtons()
{
  unsigned int i;
  
  if (orientation == HORIZONTAL) {
    // calcualte free size and stretches
    int fs = width() - 2 * _border;
    int stretch = 0;
    for (i = 0; i < buttons.count(); i++) {
      KButtonBoxItem *item = buttons.at(i);
      if (item->button != 0) {
        if (i == buttons.count() - 1)
          fs -= item->button->width();
        else
          fs -= item->button->width() + _autoborder;
      } else
        stretch += item->stretch;
    }
    
    // distribute buttons
    int x_pos = _border;
    for (i = 0; i < buttons.count(); i++) {
      KButtonBoxItem *item = buttons.at(i);
      if (item->button != 0) {
        QPushButton *b = item->button;
/* KK        if (b->style() == MotifStyle && b->isDefault()) { */
        if (0) {
          b->move(x_pos + extraMotifWidth / 2,
                  (height() - b->height()) / 2 +
                  extraMotifHeight / 2);
          if (_autoborder < extraMotifWidth / 2)
            x_pos += extraMotifWidth;
        } else
          b->move(x_pos, (height() - b->height()) / 2);
        
        x_pos += b->width() + _autoborder;
      } else
        x_pos += (int) ((((double) fs) * item->stretch) / stretch);
    }
  } else {			// VERTICAL
    // calcualte free size and stretches
    int fs = height() - 2 * _border;
    int stretch = 0;
    for (i = 0; i < buttons.count(); i++) {
      KButtonBoxItem *item = buttons.at(i);
      if (item->button != 0)
        fs -= item->button->height() + _autoborder;
      else
        stretch += item->stretch;
    }
    
    // distribute buttons
    int y_pos = _border;
    for (i = 0; i < buttons.count(); i++) {
      KButtonBoxItem *item = buttons.at(i);
      if (item->button != 0) {
        QPushButton *b = item->button;
/*        if (b->style() == MotifStyle && b->isDefault()) { */
        if (0) {
          b->move((width() - b->width()) / 2 +
                  extraMotifWidth / 2,
                  y_pos + extraMotifHeight / 2);
          if (_autoborder < extraMotifHeight / 2)
            y_pos += extraMotifHeight;
        } else
          b->move((width() - b->width()) / 2, y_pos);
        
        y_pos += b->height() + _autoborder;
      } else
        y_pos += (int) ((((double) fs) * item->stretch) / stretch);
    }
  }
}

void KButtonBox::resizeEvent(QResizeEvent *)
{
  placeButtons();
}

QSize KButtonBox::bestButtonSize() const
{
  QSize s(0, 0);
  unsigned int i;
  
  // calculate optimal size
  for (i = 0; i < buttons.count(); i++) {
    KButtonBox *that = (KButtonBox *) this;	// to remove the const ;(
    KButtonBoxItem *item = that->buttons.at(i);
    QPushButton *b = item->button;
    
    if (b != 0 && !item->noexpand) {
      QSize bs = buttonSizeHint(b);
      
      if (bs.width() > s.width())
        s.setWidth(bs.width());
      if (bs.height() > s.height())
        s.setHeight(bs.height());
    }
  }
  
  return s;
}

QSize KButtonBox::sizeHint() const
{
  unsigned int i, dw;
  bool hasMotifDefault = FALSE;

  if (buttons.count() == 0)
    return QSize(0, 0);
  else {
    dw = 2 * _border;
    
    QSize bs = bestButtonSize();
    for (i = 0; i < buttons.count(); i++) {
      KButtonBox *that = (KButtonBox *) this;
      KButtonBoxItem *item = that->buttons.at(i);
      QPushButton *b = item->button;
      if (b != 0) {
/*        hasMotifDefault |= (style() == MotifStyle) && (b->isDefault()); */
        
        QSize s;
        if (item->noexpand)
          s = that->buttonSizeHint(b);
        else
          s = bs;
        
        if (orientation == HORIZONTAL)
          dw += s.width();
        else
          dw += s.height();
        
        if (i != buttons.count() - 1)
          dw += _autoborder;
      }
    }
    
    if (orientation == HORIZONTAL) {
/* KK      if (style() == MotifStyle && hasMotifDefault) */
      if (0)
        return QSize(dw + extraMotifWidth,
                     bs.height() + 2 * _border + extraMotifHeight);
      else
        return QSize(dw, bs.height() + 2 * _border);
    } else {
/* KK      if (style() == MotifStyle) */
      if (0)
        return QSize(bs.width() + 2 * _border + extraMotifWidth,
                     dw + extraMotifHeight);
      else
        return QSize(bs.width() + 2 * _border, dw);
    }
  }
}

/*
 * Returns the best size for a button. If a button is less than
 * minButtonWidth pixels wide, return minButtonWidth pixels
 * as minimum width
 */
QSize KButtonBox::buttonSizeHint(QPushButton * b) const
{
  QSize s = b->sizeHint();
  QSize ms = b->minimumSize();
  if (s.width() < minButtonWidth)
    s.setWidth(minButtonWidth);
  
  // allows the programmer to override the settings
  if (ms.width() > s.width())
    s.setWidth(ms.width());
  if (ms.height() > s.height())
    s.setHeight(ms.height());
  
  return s;
}

KSeparator::KSeparator(QWidget * parent, const char *name, WFlags f)
  :  QFrame(parent, name, f)
{
  this->setLineWidth(1);
  this->setMidLineWidth(0);
  setOrientation(HLine);
}

KSeparator::KSeparator(int orientation, QWidget * parent, const char *name,
		       WFlags f)
  :QFrame(parent, name, f)
{
  setLineWidth(1);
  setMidLineWidth(0);
  setOrientation(orientation);
}

void KSeparator::setOrientation(int orientation)
{
  switch (orientation) {
  case VLine:
    setFrameStyle(QFrame::VLine | QFrame::Sunken);
    setMinimumSize(2, 0);
    break;
    
  default:
    warning("KSeparator::setOrientation(): invalid orientation, using default orientation HLine");
    
  case HLine:
    setFrameStyle(QFrame::HLine | QFrame::Sunken);
    setMinimumSize(0, 2);
    break;
  }
}

int KSeparator::orientation() const
{
  if (frameStyle() & VLine)
    return VLine;

  if (frameStyle() & HLine)
    return HLine;
  
  return 0;
}

QSize KSeparator::sizeHint() const
{
  if (frameStyle() & VLine)
    return QSize(2, 0);
  
  if (frameStyle() & HLine)
    return QSize(0, 2);
  
  return QSize(-1, -1);
}

#include <qapp.h>
#include <qfont.h>
#include <qstring.h>
/* #include <qwindowsstyle.h> */
#include <iostream>
#include "isdn.h"

int main(int argc, char *argv[])
{
  setlocale(LC_ALL, "");
  bindtextdomain(PACKAGE, LOCALEDIR);
  textdomain(PACKAGE);
  
  QApplication a(argc, argv);
  
//   a.setStyle(new QWindowsStyle());
  a.setFont(QFont("Helvetica", 12));
  
  KIsdnConfig *kisdnconfig = new KIsdnConfig();
  
  KCDialog *kcd = new KCDialog(kisdnconfig, 0, "isdn-config", TRUE);
  kcd->setFixedSize(kcd->minimumSize());
  kcd->setCaption("Red Hat ISDN config");
  
  return kcd->exec();
}
#endif
