/* Copyright 2000 Red Hat, Inc.
 *
 * Than Ngo <than@redhat.de>
 *
 * This software may be freely redistributed under the terms of the GNU
 * public license.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


#include <iostream>

#ifndef QTONLY
#  include <klocale.h>
#  include <kiconloader.h>
#else
#  include "kde-int.h"
#endif

#include "generaldata.h"
#include "providerinfo.h"
#include <qregexp.h>

ProviderInfo::ProviderInfo(QWidget * parent, const char *name)
	: QDialog(parent, name, true)
{
  provideredited = false;
  filldialog();
}

void ProviderInfo::slotauthentication(int _index)
{
  bool t = _index == GeneralData::AUTH_NONE ? false : true;
  loginname->setEnabled(t);
  password->setEnabled(t);
}

void ProviderInfo::slotcallback()
{
  if (callback->isChecked()) {
    phone_in->setEnabled(true);
    phone_in->setFocus();
    cbdelay->setValue(3);
    cbdelay->setEnabled(true);
  }
  else {
    phone_in->setEnabled(false);
    cbdelay->setEnabled(false);
  }
}

void ProviderInfo::slotlocalIP()
{
  if (localip_button->isChecked()) {
    localip->setEnabled(TRUE);
    localip->setFocus();
  } 
  else
    localip->setEnabled(FALSE);
}

void ProviderInfo::slotremoteIP()
{
  if (remoteip_button->isChecked()) {
    remoteip->setEnabled(TRUE);
    remoteip->setFocus();
  }
  else
    remoteip->setEnabled(FALSE);
}

bool ProviderInfo::isproviderexisted()
{
  if (ISDNData.isproviderexisted(providername->text())) {
    providerexistmessage();
    return true;
  }
  return false;
}

void ProviderInfo::noprovidernamemessage()
{
  QMessageBox::warning(this, i18n("No provider name"),
                       i18n("You must enter the name for your\ninternet provider.\n\n"));
}

void ProviderInfo::invalidprovidernamemessage()
{
  QMessageBox::warning(this, i18n("invalid provider name"),
                       i18n("The provider name contains invalid\ncharacters.\n\n"));
}

void ProviderInfo::providerexistmessage()
{
  QMessageBox::warning(this, i18n("Providername exists"),
                       i18n("The provider name exists already.\n"
                            "Please choose another provider name.\n\n"));
}

void ProviderInfo::providerinfoincompletemessage()
{
  QMessageBox::warning(this, i18n("Providerinfo is incomplete"),
                       i18n("The providerinfo is incomplete.\n"
                            "Please complete the providerinfo.\n\n"));
}

bool ProviderInfo::isprovidernamecorrect()
{
  QString s = providername->text();
  QString substring = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789-_";

  // Providername is empty
  if (s.isEmpty()) {
    noprovidernamemessage();
    return FALSE;
  }
  // check valid Character in String
  for (uint i = 0; i < s.length(); i++) {
    if (!substring.contains(s.at(i))) {
      invalidprovidernamemessage();
      return FALSE;
    }
  }
  return TRUE;
}

bool ProviderInfo::isproviderinfocorrect()
{
  if (!isprovidernamecorrect())
    return false;
  if (!provideredited && isproviderexisted())
    return false;
  if (phone->text().isEmpty() || msn->text().isEmpty()) {
    providerinfoincompletemessage();
    return false;
  }
  if (auth->currentItem() != GeneralData::AUTH_NONE) {
    if (loginname->text().isEmpty() || password->text().isEmpty()) {
      providerinfoincompletemessage();
      return false;
    }
  }
  if (localip_button->isChecked() && localip->text().isEmpty()) {
    providerinfoincompletemessage();
    return false;
  }
  if (remoteip_button->isChecked() && remoteip->text().isEmpty()) {
    providerinfoincompletemessage();
    return false;
  }
  return true;
}

void ProviderInfo::accept()
{
  if (!isproviderinfocorrect())
    return;

  ProviderData *t = new ProviderData();

  // Provider
  t->providername = providername->text();
  t->areacode = areacode->text();
  t->phonenumber = phone->text();
  // remove suffix in String 
  t->huptimeout = huptimeout->text().replace(QRegExp(huptimeout->suffix()), "").toUInt();
  t->msn = msn->text();

  if (encap->currentItem() == GeneralData::SYNCPPP)
    t->encapsulation = "syncppp";
  else if (encap->currentItem() == GeneralData::RAWIP)
    t->encapsulation = "rawip";
  
  if (dialmode->currentItem() == AUTO) {
    t->dialmode = "auto";
  }
  else if (dialmode->currentItem() == MANUAL)
    t->dialmode = "manual";

  t->isproviderenabled = activeprovider->isChecked();
  t->callback = callback->isChecked() ? "out" : "off";
  t->phone_in = phone_in->isEnabled() ? phone_in->text() : QString("");
  t->secure = secure->isChecked() ? "on" : "off";
  t->bundling = bundling->isChecked() ? "on" : "off";

  // Account
  switch (auth->currentItem())
    {
    case GeneralData::AUTH_PAP:
      t->auth = "+pap -chap";
      break;
    case GeneralData::AUTH_CHAP:
      t->auth = "-pap +chap";
      break;
    case GeneralData::AUTH_BOTH:
      t->auth = "+pap +chap";
      break;
    case GeneralData::AUTH_NONE:
      t->auth = "-pap -chap";
      break;
    }

  t->loginname = loginname->isEnabled() ? loginname->text() : QString("");
  t->password = password->isEnabled() ? password->text() : QString("");

  // Network
  t->dns = dns->text();
  t->domain = domain->text();
  t->islocalipenabled = localip_button->isChecked();
  t->isremoteipenabled = remoteip_button->isChecked();
  t->localip = localip_button->isChecked() ? localip->text() : QString("");
  t->remoteip = remoteip_button->isChecked() ? remoteip->text() : QString("");

  // adds new provider in Qlist
  if (provideredited) {
    int _index = ISDNData.providerdata.at();
    ISDNData.providerdata.remove();
    ISDNData.providerdata.insert(_index, t);
  }
  else
    ISDNData.providerdata.append(t);
  
  done(Accepted);
}

void ProviderInfo::fillproviderinfo(const QString &s)
{
  provideredited = true;
  ProviderData *t;

  // take current provider from Qlist
  for (t = ISDNData.providerdata.first(); t != 0;
       t = ISDNData.providerdata.next()) {
    if (t->providername == s)
      break;
  }

  // Provider
  providername->setText(t->providername);
  areacode->setText(t->areacode);
  phone->setText(t->phonenumber);
  huptimeout->setValue(t->huptimeout);
  huptimeout->setSuffix(i18n(" seconds"));
  msn->setText(t->msn);

  if (t->encapsulation.lower() == QString("syncppp"))
    encap->setCurrentItem(GeneralData::SYNCPPP);
  else
    encap->setCurrentItem(GeneralData::RAWIP);

  if (t->dialmode.lower() == QString("manual"))
    dialmode->setCurrentItem(MANUAL);
  else
    dialmode->setCurrentItem(AUTO);

  callback->setChecked(t->iscallbackenabled());
  if (t->iscallbackenabled()) {
    phone_in->setText(t->phone_in);
    phone_in->setEnabled(true);
  }
  else
    phone_in->setEnabled(false);

  secure->setChecked(t->issecureenabled());

  oldisproviderenabled = t->isproviderenabled;
  activeprovider->setChecked(t->isproviderenabled);
  bundling->setChecked(t->isbundlingenabled());

  // Account
  if (t->encapsulation.lower() == QString("syncppp")) {
    accountgbox->setEnabled(true);
    loginname->setText(t->loginname);
    password->setText(t->password);
    if (QString("+pap -chap") == t->auth.lower()
        || QString("-chap +pap") == t->auth.lower())
      auth->setCurrentItem(GeneralData::AUTH_PAP);
    else if (QString("-pap +chap") == t->auth.lower()
             || QString("+chap-pap") == t->auth.lower())
      auth->setCurrentItem(GeneralData::AUTH_CHAP);
    else if (QString("+pap +chap") == t->auth.lower()
             || QString("+chap +pap") == t->auth.lower())
      auth->setCurrentItem(GeneralData::AUTH_BOTH);
    else if (QString("-pap -chap") == t->auth.lower()
             || QString("-chap -pap") == t->auth.lower()) {
      auth->setCurrentItem(GeneralData::AUTH_NONE);
      loginname->setEnabled(FALSE);
      password->setEnabled(FALSE);
    }
  }
  else {
    auth->setCurrentItem(GeneralData::AUTH_NONE);
    accountgbox->setEnabled(false);
  }
  
  // Network
  dns->setText(t->dns);
  domain->setText(t->domain);
  localip_button->setChecked(t->islocalipenabled);
  localip->setText(t->localip);
  localip->setEnabled(t->islocalipenabled);
  remoteip_button->setChecked(t->isremoteipenabled);
  remoteip->setText(t->remoteip);
  remoteip->setEnabled(t->isremoteipenabled);
}

bool ProviderInfo::haslocalip()
{
  return localip->isEnabled();
}

bool ProviderInfo::hasremoteip()
{
  return remoteip->isEnabled();
}

void ProviderInfo::slotencapsulation(int i)
{
  if (i == GeneralData::RAWIP) {
    auth->setCurrentItem(GeneralData::AUTH_NONE);
    accountgbox->setEnabled(false);
    localip->setEnabled(true); // rawip requires localip and remoteip
    localip_button->setChecked(true);
    remoteip->setEnabled(true);
    remoteip_button->setChecked(true);
  }
  else {
    auth->setCurrentItem(GeneralData::AUTH_PAP);
    accountgbox->setEnabled(true);
    localip->setEnabled(false);	// disable localip and remoteip
    localip_button->setChecked(false);
    remoteip->setEnabled(false);
    remoteip_button->setChecked(false);
  }
}

void ProviderInfo::filldialog()
{
  //setFixedHeight(minimumHeight());
  setCaption(i18n("New provider"));
  setFocusPolicy(StrongFocus);

  QBoxLayout *toplayout = new QVBoxLayout(this, 8);
  QBoxLayout *layout1 = new QHBoxLayout(toplayout);

  // Provider
  providergbox = new QGroupBox(2, Qt::Horizontal, i18n("Provider"), this);
  layout1->addWidget(providergbox);

  QLabel *label = new QLabel(i18n("&Provider name:"), providergbox);
  providername = new QLineEdit(providergbox);
  QToolTip::add(providername,
                i18n("The name of this configuration, usualy just\n"
                     "the name of the ISP"));
  label->setBuddy(providername);
  providername->setFocus();

  label = new QLabel(i18n("&Area code:"), providergbox);
  areacode = new QLineEdit(providergbox);
  QToolTip::add(areacode, i18n("The area code of the ISP"));
  connect(areacode, SIGNAL(textChanged(const QString &)),
          SLOT(areacodeChange(const QString &)));
  label->setBuddy(areacode);

  label = new QLabel(i18n("P&hone number:"), providergbox);
  phone = new QLineEdit(providergbox);
  QToolTip::add(phone, i18n("The phone number of the ISP (without area code)"));
  connect(phone, SIGNAL(textChanged(const QString &)),
          SLOT(phoneChange(const QString &)));
  label->setBuddy(phone);

  label = new QLabel(i18n("HUP tim&eout:"), providergbox);
  huptimeout = new QSpinBox(0, 1200, 10, providergbox);
  QToolTip::add(huptimeout, i18n("Set the hangup timeout for ISDN interface name to seconds"));
  huptimeout->setValue(600);
  huptimeout->setSuffix(i18n(" seconds"));
  label->setBuddy(huptimeout);

  label = new QLabel(i18n("M&SN/EAZ:"), providergbox);
  msn = new QLineEdit(providergbox);
  QToolTip::add(msn, i18n("Set MSN (Euro-ISDN  E-DSS1) or the\n"
                          "EAZ (German 1TR6 protocol) for ISDN\n"
                          "interface name"));
  connect(msn, SIGNAL(textChanged(const QString &)),
          SLOT(msnChange(const QString &)));
  label->setBuddy(msn);

  label = new QLabel(i18n("E&ncapsulation:"), providergbox);
  encap = new QComboBox(providergbox);
  QToolTip::add(encap,
                i18n("Set the encapsulation mode for interface name"));
  connect(encap, SIGNAL(activated(int)), SLOT(slotencapsulation(int)));
  label->setBuddy(encap);
  encap->insertStrList(GeneralData::encap_items);

  label = new QLabel(i18n("D&ialmode:"), providergbox);
  dialmode = new QComboBox(providergbox);
  QToolTip::add(dialmode, i18n("Set the dial mode of the interface"));
  label->setBuddy(dialmode);
  dialmode->insertItem(i18n("Auto"));
  dialmode->insertItem(i18n("Manual"));
  dialmode->setCurrentItem(1); // set dialmode manual as dedault

  callback = new QCheckBox(i18n("&Callback:"), providergbox);
  callback->setChecked(false);
  QToolTip::add(callback, i18n("Enable or disable Callback"));
  phone_in = new QLineEdit(providergbox);
  phone_in->setEnabled(false);
  QToolTip::add(phone_in, i18n("Set the in coming phone number"));
  connect(callback, SIGNAL(clicked()), SLOT(slotcallback()));

  label = new QLabel(i18n("Callbac&k delay:"), providergbox);
  cbdelay = new QSpinBox(0, 10, 1, providergbox);
  cbdelay->setValue(3);
  cbdelay->setSuffix(i18n(" seconds"));
  QToolTip::add(cbdelay, i18n("Set the callback delay"));
  label->setBuddy(cbdelay);
  cbdelay->setEnabled(false);
  
  secure = new QCheckBox(i18n("Securit&y"), providergbox);
  secure->setChecked(false); // set security off as default
  QToolTip::add(secure, i18n("Enable oder disable the security"));
  providergbox->addSpace(0);

  bundling = new QCheckBox(i18n("Cha&nnel bundling"), providergbox);
  bundling->setChecked(false);
  QToolTip::add(bundling, i18n("Enable or disable the channel bundling"));
  providergbox->addSpace(0);

  activeprovider = new QCheckBox(i18n("Pro&vider"), providergbox);
  activeprovider->setChecked(true);
  QToolTip::add(activeprovider, i18n("Enable or disable the provider to dialout"));
  providergbox->addSpace(0);

  // Account
  QBoxLayout *vlayout = new QVBoxLayout(layout1);
  accountgbox = new QGroupBox(2, Qt::Horizontal, i18n("Account"), this);
  vlayout->addWidget(accountgbox);

  label = new QLabel(i18n("A&uthentication:"), accountgbox);
  auth = new QComboBox(accountgbox);
  QToolTip::add(auth, i18n("Set the authentication protocol used by the ISP"));
  label->setBuddy(auth);
  auth->insertStrList(GeneralData::auth_items);
  connect(auth, SIGNAL(activated(int)), SLOT(slotauthentication(int)));

  label = new QLabel(i18n("&Login name:"), accountgbox);
  loginname = new QLineEdit(accountgbox);
  QToolTip::add(loginname, i18n("Set the login name to authenticate the user"));
  label->setBuddy(loginname);

  label = new QLabel(i18n("Pass&word:"), accountgbox);
  password = new QLineEdit(accountgbox);
  QToolTip::add(password, i18n("Set the password to authenticate the user"));
  password->setEchoMode(QLineEdit::Password);
  label->setBuddy(password);

  // Network
  networkgbox = new QGroupBox(2, Qt::Horizontal, i18n("Network"), this);
  vlayout->addWidget(networkgbox);

  label = new QLabel(i18n("&DNS:"), networkgbox);
  dns = new QLineEdit(networkgbox);
  QToolTip::add(dns, i18n("Set the nameserver of the ISP"));
  label->setBuddy(dns);

  label = new QLabel(i18n("Do&main name:"), networkgbox);
  domain = new QLineEdit(networkgbox);
  QToolTip::add(domain, i18n("Set the domain of the ISP"));
  label->setBuddy(domain);

  localip_button = new QCheckBox(i18n("L&ocal IP:"), networkgbox);
  QToolTip::add(localip_button, i18n("Enable or diable the local IP"));
  localip = new QLineEdit(networkgbox);
  QToolTip::add(localip, i18n("Set the local IP. Only needed with static IP negotiation"));
  localip->setEnabled(FALSE);
  connect(localip_button, SIGNAL(clicked()), SLOT(slotlocalIP()));

  remoteip_button = new QCheckBox(i18n("&Remote IP:"), networkgbox);
  QToolTip::add(remoteip_button, i18n("Enable or diable the remote IP"));
  remoteip = new QLineEdit(networkgbox);
  QToolTip::add(remoteip, i18n("Set the remote IP. Only needed with static IP negotiation"));
  remoteip->setEnabled(FALSE);
  connect(remoteip_button, SIGNAL(clicked()), SLOT(slotremoteIP()));

  // Horizontal Line
  QFrame *tmpQFrame = new QFrame(this);
  tmpQFrame->setFrameStyle(QFrame::HLine | QFrame::Sunken);
  toplayout->addWidget(tmpQFrame);

  // Button Ok, Cancel
  KButtonBox *bbox = new KButtonBox(this);
  bbox->addStretch(10);

  QPushButton *ok = bbox->addButton(i18n("OK"));
  connect(ok, SIGNAL(clicked()), SLOT(accept()));

  QPushButton *cancel = bbox->addButton(i18n("Cancel"));
  connect(cancel, SIGNAL(clicked()), SLOT(reject()));

  bbox->layout();
  toplayout->addWidget(bbox);
}

void ProviderInfo::msnChange(const QString & s)
{
}

void ProviderInfo::phoneChange(const QString & s)
{
}

void ProviderInfo::areacodeChange(const QString & s)
{
}

ProviderInfo::~ProviderInfo()
{
}
